/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : 28.03.04
    copyright            : (C) 2004 by Pablo
    email                : rex@supertux.homelinux.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "Xserver.h"

#define MAX_LENGTH 4096
#define FTW_F   1 /* file is not a directory */
#define FTW_D   2 /* file is a directory */
#define FTW_DNR 3 /* no readable directory */
#define FTW_NS  4 /* file does't function with stat */

#define MAX_PATH 500

typedef struct TREE_S {
  char name[MAX_LENGTH];
  int id;
  int empty;
  struct TREE_S* next;
} TREE;

TREE* createTREE(TREE** tree)
{
    TREE* tmp;
    if(tree==NULL) return *tree=NULL;
    tmp = (TREE*) malloc(sizeof(TREE));
    if (!tmp) {
      fprintf(stderr, "Error allocation memory\n");
      return *tree=NULL;
    }
    tmp->empty=1;
    tmp->next=NULL;
    return *tree=tmp;    
}

void killTREE(TREE* tree)
{
    TREE* tmp=tree, *next;
    while(tmp->next!=NULL) {
       next=tmp->next;
       free(tmp);
       tmp=next;
    }
}

int haveName(TREE* tree, const char* name)
{
  TREE* tmp=tree;
  if (!tree || !name) return 0;
  
  while(tmp!=NULL) {
       	if (!strcmp(tmp->name, name)) return 1;
       	tmp=tmp->next;
    	if(!tmp) return 0;
  }
  
  return 0;
}
					    

char* getname(TREE* tree, int id)
{
    TREE* tmp=tree;
    if (!tree) return NULL;

    while(tmp!=NULL) {
    	if (tmp->id==id) return tmp->name;
      	tmp=tmp->next;
	if(!tmp) return NULL;
    } 
    
    return NULL;				 
}

int addElement(TREE* tree, const char* name, int id)
{
    TREE* tmp;
    TREE* run=tree;
    TREE* loc;
    
    if (!tree || !name) return 0;

    // is this an empty tree?
    if (tree->next==NULL && tree->empty) {
        strcpy(tree->name, name);
	tree->id=id;
	tree->empty=0;
	return 1;
    }

    // move to the last node
    while(run->next!=NULL) {
    	loc = run->next;
	run = loc;
    }

    tmp=createTREE(&tmp);
    strcpy(tmp->name, name);
    tmp->id=id;
    if (!tmp) return 0;
   
    run->next=tmp;

    return 1;    
}

void parse_args(int argc, char** argv)
{
  int i;
  for(i=0; i<argc; ++i) {
    if(!strcmp(argv[i], "--help")) {
    	printf("usage: %s [path1] [path2] ... [--help] [--version]\n", argv[0]);
	exit(EXIT_SUCCESS);
    }
    if(!strcmp(argv[i], "--version")) {
    	printf("genbg version 1.0.1\n");
	exit(EXIT_SUCCESS);
    }
    if(argv[i][0]=='-') {
    	fprintf(stderr,"The argument %s is not valid\n", argv[i]);
	exit(EXIT_FAILURE);
    }
  }
}

static int isPicture(const char* path)
{
    char* formats[] = { ".jpg", ".gif", ".bmp", ".png", ".xpm" }, *tmp=path;
    int i, j;
    if (!path) return 0;
    if (strlen(path) < 5) return 0; // mindestsns .xyz
    tmp+=strlen(path)-4;
    j=0;
    for (i=0; i<5; ++i) 
        j += !strcmp(formats[i], tmp);
    
    return j;
}

void add_files(TREE* tree, const char* dir, int* id, int root)
{
    DIR* directory;
    struct dirent* dirInfo;
    char buffer[MAX_LENGTH], text[MAX_LENGTH], curr[MAX_LENGTH];
    int i;

        
    if (!tree || !dir || !id) return;
    
    buffer[0]=0;
    for(i=0; i<root; ++i)
       strcat(buffer, " ");

    sprintf(text, "%s", dir);
    strcat(buffer, text);
    printf(buffer);
    if (NULL==(directory=opendir(dir))) {
       printf(" (cannot be opened)\n");
       return;
    }
    printf(" (opened)\n");
    buffer[0]=0;
    for(i=0; i<root+2; ++i)
       strcat(buffer, " ");

    while ((dirInfo = readdir(directory)) != NULL) {
	// 4 is directory, 8 is file
	if(strcmp(dirInfo->d_name, ".") && strcmp(dirInfo->d_name, "..") && strcmp(dirInfo->d_name, ".xvpics")) {
	    /* .xvpics for The GIMP */
	    if (dirInfo->d_type==4) {
	        strcpy(curr, dir);
		if (curr[strlen(curr)-1]!='/') strcat(curr, "/");
		strcat(curr, dirInfo->d_name);
	        add_files(tree, curr, id, root+2);
	    }
	    else {
	        if (isPicture(dirInfo->d_name)) {
		    strcpy(curr, dir);
		    if (curr[strlen(curr)-1]!='/') strcat(curr, "/");
		    strcat(curr, dirInfo->d_name);
		    if (!haveName(tree, curr)) {
		        if (addElement(tree, curr, ++(*id))) 
			    printf("%s%s (added)\n", buffer, curr);
			else --(*id);
		    }
		}
	    }
	}
    }
    closedir(directory);
}

int generateRandom(int left, int right)
{
    int z;
    if (left > right) return 0;
    if (left==right) return left;
    
    srand(time(NULL));

    z = rand()%right+1;

    z += z<left ? left : 0;

    return z;
}

int main(int argc, char** argv)
{
    Display* display;
    int i,id, height, width;
    TREE* tree;
    char command [MAX_LENGTH], *tmp;

    //parsing arguemnt
    parse_args(argc, argv);

    display = NULL;
    display = connectX(display);
    get_size(display, &width, &height);

    
    tree = createTREE(&tree);
    if (!tree) return EXIT_SUCCESS;


    id=1;
    if (argc==1) 
    	add_files(tree, ".", &id,0);
    for(i=1; i<argc; ++i)
    	add_files(tree, argv[i], &id,0);

    --id;

    if (tree->empty) {
    	fprintf(stderr, "There are no files that has been added\n");
	killTREE(tree);
 	closeX(display);
	return EXIT_FAILURE;
    }

    printf("%d files added into the database.\n", id);
    
    do {
    	i=generateRandom(1, id);
        tmp = getname(tree, i);
    } while (tmp==NULL);

    printf("Selected picture: %s (id=%d)\n", tmp, i);


    sprintf(command, "wmsetbg \"%s\"", tmp);
    printf("Executing... %s\n", command);
    system(command);

    killTREE(tree);
    closeX(display);
    return EXIT_SUCCESS;
}
